@@ -14,9 +14,10 @@ from pathlib import Path
 from types import SimpleNamespace
 import zipfile
 from pathlib import Path
+import sysconfig
 
 from . import common
-from .__main__ import __version__
+from .templates import __version__
 from .xmake import XMaker
 from .builder.wheel_tag import WheelTag
 
@@ -83,11 +84,13 @@ class WheelBuilder:
         if target_fp is not None:
             self.wheel_zip = zipfile.ZipFile(target_fp, 'w',
                                  compression=zipfile.ZIP_DEFLATED)
-        self.xmake = xmake
+        self.temp = tempfile.TemporaryDirectory()
+        self.root = Path(self.temp.name)
+        self.data = self.root / "data"
         if xmake:
-            self.output = xmake.output
-        else:
-            self.output = "."
+            xmake.tempname = self.temp.name
+        self.xmake = xmake
+        self.kind = 0
 
     @classmethod
     def from_ini_path(cls, ini_path, target_fp):
@@ -102,9 +105,10 @@ class WheelBuilder:
         xmake_path = directory / "xmake.lua"
         if xmake_path.exists() or xmaker != {}:
             xmake = XMaker(xmaker.get("xmake", "xmake"),
-                           xmaker.get("root", "."),
-                           xmaker.get("out", "."),
-                           xmaker.get("command", ""))
+                           xmaker.get("command", ""),
+                           xmaker.get("tempname", ""),
+                           xmaker.get("project", os.path.abspath(".")),
+                           ini_info.metadata["version"])
         return cls(
             directory, module, metadata, entrypoints, target_fp, ini_info.data_directory, xmake
         )
@@ -116,10 +120,14 @@ class WheelBuilder:
     @property
     def wheel_filename(self):
         dist_name = common.normalize_dist_name(self.metadata.name, self.metadata.version)
-        if self.xmake:
-            tag = str(WheelTag.compute_best([]))
-        else:
-            tag = ('py2.' if self.metadata.supports_py2 else '') + 'py3-none-any'
+        py_api = ""
+        root_is_purelib = True
+        if self.kind != 0:
+            root_is_purelib = False
+        if self.kind == 1:
+            py_api = ('py2.' if self.metadata.supports_py2 else '') + 'py3'
+        archs = [sysconfig.get_platform().split("-")[-1]]
+        tag = str(WheelTag.compute_best(archs, py_api, root_is_purelib=root_is_purelib))
         return '{}-{}.whl'.format(dist_name, tag)
 
     def _add_file(self, full_path, rel_path):
@@ -182,10 +190,9 @@ class WheelBuilder:
 
     def copy_module(self):
         log.info('Copying package file(s) from %s', self.module.path)
-        source_dir = str(self.module.source_dir)
 
-        for full_path in self.module.iter_files():
-            rel_path = osp.relpath(full_path, source_dir)
+        for full_path in common.walk_data_dir(self.root / "platlib"):
+            rel_path = os.path.relpath(full_path, self.root / "platlib")
             self._add_file(full_path, rel_path)
 
     def add_pth(self):
@@ -199,25 +206,29 @@ class WheelBuilder:
         for full_path in common.walk_data_dir(self.data_directory):
             rel_path = os.path.relpath(full_path, self.data_directory)
             self._add_file(full_path, dir_in_whl + rel_path)
-        for name in {"lib", "share"}:
-            for full_path in common.walk_data_dir(str(Path(self.output) / name)):
-                rel_path = os.path.relpath(full_path, self.data_directory).partition(name + os.path.sep)[2]
+        if not self.data.exists():
+            return
+        for name in os.listdir(self.data):
+            if name in {"bin", "include"}:
+                continue
+            for full_path in common.walk_data_dir(self.data / name):
+                rel_path = os.path.relpath(full_path, self.data).partition(name + os.path.sep)[2]
                 self._add_file(full_path, dir_in_whl + name + os.path.sep + rel_path)
 
     def add_scripts_directory(self):
         dir_in_whl = '{}.data/scripts/'.format(
             common.normalize_dist_name(self.metadata.name, self.metadata.version)
         )
-        for full_path in common.walk_data_dir(str(Path(self.output) / "bin")):
-            rel_path = os.path.relpath(full_path, self.data_directory).partition("bin" + os.path.sep)[2]
+        for full_path in common.walk_data_dir(Path(self.data) / "bin"):
+            rel_path = os.path.relpath(full_path, Path(self.data) / "bin")
             self._add_file(full_path, dir_in_whl + rel_path)
 
     def add_headers_directory(self):
         dir_in_whl = '{}.data/headers/'.format(
             common.normalize_dist_name(self.metadata.name, self.metadata.version)
         )
-        for full_path in common.walk_data_dir(str(Path(self.output) / "include")):
-            rel_path = os.path.relpath(full_path, self.data_directory).partition("include" + os.path.sep)[2]
+        for full_path in common.walk_data_dir(Path(self.data) / "include"):
+            rel_path = os.path.relpath(full_path, Path(self.data) / "include")
             self._add_file(full_path, dir_in_whl + rel_path)
 
     def write_metadata(self):
@@ -229,6 +240,9 @@ class WheelBuilder:
 
         for file in self.metadata.license_files:
             self._add_file(self.directory / file, '%s/licenses/%s' % (self.dist_info, file))
+        for full_path in common.walk_data_dir(self.root / "metadata"):
+            rel_path = os.path.relpath(full_path, self.root / "metadata")
+            self._add_file(full_path, '%s/%s' % (self.dist_info, rel_path))
 
         with self._write_to_zip(self.dist_info + '/WHEEL') as f:
             _write_wheel_file(f, supports_py2=self.metadata.supports_py2)
@@ -246,23 +260,25 @@ class WheelBuilder:
             f.write(self.dist_info + '/RECORD,,\n')
 
     def build(self, editable=False):
-        if self.xmake:
-            self.xmake.config()
-            self.xmake.build()
-            self.xmake.install()
-        try:
-            if editable:
-                self.add_pth()
-            else:
-                self.copy_module()
-            self.add_data_directory()
-            self.add_scripts_directory()
-            self.add_headers_directory()
-            self.write_metadata()
-            self.write_record()
-        finally:
-            if self.wheel_zip:
-                self.wheel_zip.close()
+        with self.temp:
+            if self.xmake:
+                self.xmake.config()
+                self.xmake.build()
+                self.xmake.install()
+                self.kind = self.xmake.show()
+            try:
+                if editable:
+                    self.add_pth()
+                else:
+                    self.copy_module()
+                self.add_data_directory()
+                self.add_scripts_directory()
+                self.add_headers_directory()
+                self.write_metadata()
+                self.write_record()
+            finally:
+                if self.wheel_zip:
+                    self.wheel_zip.close()
 
 def make_wheel_in(ini_path, wheel_directory, editable=False):
     # We don't know the final filename until metadata is loaded, so write to
